# Repro Flutter Plugin

- [Repro](https://repro.io/) plugin for Flutter.
- Support all the features of Repro except for WebView event tracking.

## How to add Repro Flutter Plugin to your app

### Add Repro Flutter Plugin

#### 1. Add dependencies

Add `repro_flutter` to your `pubspec.yaml` and run `flutter packages get`:

```yaml
dependencies:
  flutter:
    sdk: flutter
  ...
  repro_flutter: ^2.1.0 # add this line
```

#### 2. Install the native library (Android)

Add dependencies to `android/app/build.gradle`:

```gradle
dependencies {
    implementation 'io.repro:repro-android-sdk:5.6.3' // add this line
    testImplementation 'junit:junit:4.12'
    androidTestImplementation 'com.android.support.test:runner:1.0.2'
    androidTestImplementation 'com.android.support.test.espresso:espresso-core:3.0.2'
}
```

#### 3. Install the native library (iOS)

Run the following command in the root directory of your app.

```sh
$ cd ios
$ pod install
```

#### 4. Add native code (Android)

Add a custom `Application` class extending from `io.flutter.app.FlutterApplication`. If you've done this before for the other reasons, please skip this step and just add the call to `Repro.setup` (in below).

Create a new `Application` class under `android/app/src/main/your/package/name`:

```java
package your.package.name;

import io.flutter.app.FlutterApplication;
import io.repro.android.Repro;

public class MyApplication extends FlutterApplication {

    @Override
    public void onCreate() {
        super.onCreate();
    }
}
```

Set it to `android/app/src/main/AndroidManifest.xml`:

```diff
      <application
-         android:name="io.flutter.app.FlutterApplication"
+         android:name="your.package.name.MyApplication"
          android:label="repro_integration_test"
          android:icon="@mipmap/ic_launcher">
```

Call `Repro.setup` in your `Application` class (replace `YOUR SDK TOKEN` by your SDK token):

```java
    public void onCreate() {
        super.onCreate();
        Repro.setup(this, "YOUR SDK TOKEN"); // add this line
    }
```

#### 5. Add native code (iOS)

Call `Repro#setup` in `ios/Runner/AppDelegate.m` (replace `YOUR SDK TOKEN` by your SDK token):

```objc
- (BOOL)application:(UIApplication *)application
    didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
  [GeneratedPluginRegistrant registerWithRegistry:self];
  // Override point for customization after application launch.
  [Repro setup:@"YOUR SDK TOKEN"]; // add this line
  return [super application:application didFinishLaunchingWithOptions:launchOptions];
}
```

### Enable Push Notifications

To enable push notifications, you need to add the same implementation of the Android/iOS version of the Repro SDK.

#### Setup for Android

##### 1. Add Firebase Flutter Plugin

Add `firebase_core` and `firebase_messaging` to your `pubspec.yaml` and run `flutter packages get`:

```yaml
dependencies:
  flutter:
    sdk: flutter
  firebase_core: ^0.4.0 # Add this line
  firebase_messaging: ^5.0.4 # Add this line
```

Add `google-services.json` into your project.

Please see the [official document of Firebase](https://firebase.google.com/docs/flutter/setup?hl=en) for more information.

##### 2. Add implementation

Please refer to the following document of Repro:

- [Settings for FCM (Android)](https://docs.repro.io/en/dev/sdk/push-notification/setup-android.html)
- [Push Notification for Android](https://docs.repro.io/en/dev/sdk/push-notification/android.html)

Note: For the Flutter app, you don't need to add `firebase_core` and `firebase_messaging` statements into `app/build.gradle`.

#### Setup for iOS

Please refer to the following document of Repro:

- [Settings for APNs Certificate (iOS)](https://docs.repro.io/en/dev/sdk/push-notification/setup-ios.html)
- [Push Notification for iOS](https://docs.repro.io/en/dev/sdk/push-notification/ios.html)

---

## Development Guide

Please see the example code in `example/lib/main.dart` while refer to the [Development Guide of Repro](https://docs.repro.io/en/dev/sdk/index.html).

---

## How to run the example app

Replace the following `YOUR SDK TOKEN` by your SDK token (if you don't have a Repro account, [sign-up here](https://app.repro.io/suppliers/sign_up) for free).

`android/app/src/main/java/io/repro/repro_flutter_example/MyApplication.java`:

```java
public class MyApplication extends FlutterApplication {
    @Override
    public void onCreate() {
        super.onCreate();
        Repro.setup(this, "YOUR SDK TOKEN");
```

`ios/Runner/AppDelegate.m`:

```objc
- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    [GeneratedPluginRegistrant registerWithRegistry:self];
    // Override point for customization after application launch.
    [Repro setup:@"YOUR SDK TOKEN"];
```

Go to `example` directory and run:

```sh
flutter run
```

### Enable Push Notifications in the example app

#### 1. Add Firebase Flutter Plugin (for Android)

Uncomment the following code which implements Push Notifications:

`pubspec.yaml`:

```yaml
dependencies:
  flutter:
    sdk: flutter
  # To enable push notifications on Repro, uncomment the following:
  # firebase_core: ^0.4.0
  # firebase_messaging: ^5.0.4
```

`android/build.gradle`:

```gradle
    dependencies {
        classpath 'com.android.tools.build:gradle:3.2.1'

        // To enable push notifications on Repro, uncomment the following:
        // classpath 'com.google.gms:google-services:3.2.1'
    }
```

`android/app/build.gradle`:

```gradle
// To enable push notifications on Repro, uncomment the following:
// apply plugin: 'com.google.gms.google-services'
```

Add `google-services.json` into your project.

Please see the [official document of Firebase](https://firebase.google.com/docs/flutter/setup?hl=en) for more information.

#### 2. Setup APNs Certificate and FCM

Please refer to the following document of Repro:

- [Settings for APNs Certificate (iOS)](https://docs.repro.io/en/dev/sdk/push-notification/setup-ios.html)
- [Settings for FCM (Android)](https://docs.repro.io/en/dev/sdk/push-notification/setup-android.html)

#### 3. Uncomment code related to Push Notifications

Uncomment the following code which implements Push Notifications:

`android/app/src/main/AndroidManifest.xml`:

```xml
        <!-- To enable push notifications on Repro, uncomment the following: -->
        <receiver
            android:name="io.repro.android.ReproReceiver"
            android:exported="true"
            android:permission="com.google.android.c2dm.permission.SEND">
            <intent-filter>
                <action android:name="com.google.android.c2dm.intent.RECEIVE" />
                <category android:name="io.repro.repro_flutter_example" />
            </intent-filter>
        </receiver>

        <meta-data
            android:name="io.repro.android.PushNotification.ChannelId"
            android:value="io.repro.flutter.repro_test.channel_id">
        </meta-data>

        <meta-data
            android:name="io.repro.android.PushNotification.ChannelName"
            android:resource="@string/repro_channel_name">
        </meta-data>

        <meta-data
            android:name="io.repro.android.PushNotification.ChannelDescription"
            android:resource="@string/repro_channel_description">
        </meta-data>

        <meta-data
            android:name="io.repro.android.PushNotification.ShowBadge"
            android:value="true">
        </meta-data>
```

`android/app/src/main/res/values/strings.xml`:

```xml
    <!-- To enable push notifications on Repro, uncomment the following: -->
    <string name="repro_channel_name">Channel Name</string>
    <string name="repro_channel_description">Channel Description</string>
```

`android/app/src/main/java/io/repro/repro_flutter_example/MyApplication.java`:

```java
    public void onCreate() {
        super.onCreate();
        Repro.setup(this, "YOUR SDK TOKEN");

        // To enable push notifications on Repro, uncomment the following:
        Repro.enablePushNotification();
```

`ios/Runner/AppDelegate.m`:

```objc
- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    ...

    // To enable push notifications on Repro, uncomment the following:
    if (floor(NSFoundationVersionNumber) > NSFoundationVersionNumber_iOS_9_x_Max) {
        UNUserNotificationCenter *center = [UNUserNotificationCenter currentNotificationCenter];
        [center requestAuthorizationWithOptions:(UNAuthorizationOptionAlert | UNAuthorizationOptionBadge | UNAuthorizationOptionSound) completionHandler:^(BOOL granted, NSError * _Nullable error) {
        }];
        [[UIApplication sharedApplication] registerForRemoteNotifications];
    } else {
        UIUserNotificationType types = UIUserNotificationTypeBadge | UIUserNotificationTypeSound | UIUserNotificationTypeAlert;
        UIUserNotificationSettings *settings = [UIUserNotificationSettings settingsForTypes:types categories:nil];
        [[UIApplication sharedApplication] registerUserNotificationSettings:settings];
        [[UIApplication sharedApplication] registerForRemoteNotifications];
    }

    ...
}

// To enable push notifications on Repro, uncomment the following:
- (void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken
{
    [Repro setPushDeviceToken:deviceToken];
}

- (void)application:(UIApplication *)application didFailToRegisterForRemoteNotificationsWithError:(NSError *)error
{
    NSLog(@"Remote Notification Error: %@", error);
}
```

#### 4. Send Push Notifications from Repro

Please see the [Dashboard Guide of Repro](https://docs.repro.io/en/dashboard/campaign/push-notification.html).

#### NOTE (for iOS)

Because you may not be able to create the same bundle id for testing, remember to change the bundle id `io.repro.reproFlutterExample` to one you have control of.
