part of io.repro.flutter;

/// The status of the result of a fetch the RemoteConfig.
enum FetchStatus {
  success, // 0
  timeoutReached, // 1
  alreadyFetched // 2
}

/// Class for handling RemoteConfig (UX Optimizer).
class RemoteConfig {
  static RemoteConfig _instance;

  factory RemoteConfig(MethodChannel channel) {
    if (_instance == null) {
      _instance = RemoteConfig._();
    }
    return _instance;
  }

  RemoteConfig._();

  /// Set the default value of RemoteConfig based on the map.
  Future<void> setDefaultsFromMap(final Map map) async {
    await Repro._invokeMethod('remoteConfig_setDefaultsFromMap', map);
  }

  /// Set the default value of RemoteConfig based on the JSON string.
  Future<void> setDefaultsFromJsonString(final String json) async {
    await Repro._invokeMethod('remoteConfig_setDefaultsFromJsonString', json);
  }

  /// Get the value of the RemoteConfig for the specified key.
  /// example:
  /// ```dart
  /// var remoteConfigValue = await Repro.remoteConfig.get('myKey');
  /// var stringValue = remoteConfigValue.asString();
  /// ```
  Future<RemoteConfigValue> get(final String key) async {
    // Expects to return:
    // { "String": "123", "int": 123, "toString": "123" }
    var resultValue = await Repro._invokeMapMethod<String, dynamic>('remoteConfig_get', key);
    return RemoteConfigValue(resultValue);
  }

  /// Get all RemoteConfig keys and values.
  Future<Map<String, RemoteConfigValue>> getAllValues() async {
    // Map<String, Map<String, Object>>
    var values = await Repro._invokeMapMethod<String, Map<dynamic, dynamic>>('remoteConfig_getAllValues');

    var map = <String, RemoteConfigValue>{};
    values.forEach((key, value) {
      map[key] = RemoteConfigValue(value);
    });

    return map;
  }

  /// Gets all RemoteConfig keys and values starting with the specified prefix.
  Future<Map<String, RemoteConfigValue>> getAllValuesWithPrefix(final String prefix) async {
    var values = await Repro._invokeMapMethod('remoteConfig_getAllValuesWithPrefix', prefix);
    var map = <String, RemoteConfigValue>{};
    values.forEach((key, value) {
      map[key] = RemoteConfigValue(value);
    });
    return map;
  }

  /// Waiting to get the RemoteConfig value.
  /// And when the wait is complete, the callback function will be executed.
  Future<void> fetch(final int timeoutMillisecs, _ReproCallback callback) async {
    var args = [timeoutMillisecs];
    var cleanUpFunc = await Repro._startListening(
        methodName: 'startListening_remoteConfig_fetch',
        callback: callback,
        arguments: args,
        resultTypeName: 'FetchStatus');

    await Future.delayed(Duration(milliseconds: timeoutMillisecs + 1000));
    cleanUpFunc();
  }

  /// Enable the fetched RemoteConfig value.
  /// Normally used in combination with the [fetch] method.
  /// ```dart
  /// await Repro.remoteConfig.fetch(3000, (result) {
  ///   if (result != FetchStatus.timeoutReached) {
  ///     await Repro.remoteConfig.activateFetched();
  ///   }
  /// });
  /// ```
  Future<bool> activateFetched() async {
    return await Repro._invokeMethod<bool>('remoteConfig_activateFetched');
  }

  /// Get the default value set in the device.
  Future<RemoteConfigValue> getLocalDefaultValue(final String key) async {
    var resultValue = await Repro._invokeMapMethod<String, dynamic>('remoteConfig_getLocalDefaultValue', key);
    return RemoteConfigValue(resultValue);
  }

  /// Force the RemoteConfig value held by the app to be reset.
  Future<void> forceReset() async {
    await Repro._invokeMethod('remoteConfig_forceReset');
  }
}
