/// Repro plugin for Flutter
library io.repro.flutter;

import 'dart:io' show Platform;
import 'dart:async';
import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

part 'news_feed_entry.dart';
part 'remote_config.dart';
part 'remote_config_value.dart';
part 'news_feed_campaign_type.dart';

/// Log levels for [Repro.setLogLevel()].
enum LogLevel { debug, info, warn, error }

/// Genders for [Repro.setUserGender()].
enum UserGender { male, female, other }

/// Dart's callback function that is called after Dart executes Native processing.
typedef void _ReproCallback(dynamic result);

/// Function to discard an unnecessary callback function
typedef void _CancelListening();

/// When using Repro's feature, call the API via this class.
class Repro {

  static const String _REPRO_FLUTTER_BRIDGE_VERSION = '2.1.0';

  static MethodChannel _channel = const MethodChannel('io.repro.flutter')
    ..setMethodCallHandler(_methodCallHandler);

  /// Dart --> Java --> Dart callback list.
  static List<_CallbackTask> _callbacksById = [];

  /// An ID to identify the callback function, which is necessary to call
  /// the callback function set in Dart from the Java side.
  static int _nextCallbackId = 0;

  /// Method call handler (Java --> Dart)
  static Future<void> _methodCallHandler(MethodCall call) async {
    switch (call.method) {
      case 'callListener':
        var callbackId = call.arguments['id'];
        var resultValue = call.arguments['args']; // Will be passed as an argument to the callback function.

        var callbackTask = _callbacksById.firstWhere((task) => task.callbackId == callbackId);

        _ReproCallback callback = callbackTask.callbackFunction;
        String resultTypeName = callbackTask.resultTypeName; // type name for casting `resultValue`

        // Cast a callback result value as resultTypeName
        if (resultTypeName == 'FetchStatus') {
          // Convert int --> FetchStatus
          callback(FetchStatus.values[resultValue]);
        } else {
          callback(resultValue);
        }
        break;
      case 'passRuntimeValues_dart':
        var values = {
          'sub_sdk_platform': 'flutter',
          'sub_sdk_platform_version': '',
          'sub_sdk_runtime_version': Platform.version,
          'sub_sdk_bridge_version': _REPRO_FLUTTER_BRIDGE_VERSION
        };
        await _invokeMethod('passRuntimeValues_native', values);
        break;
      default:
        break;
    }
  }

  /// Set Java --> Dart callback function and call a Java side method.
  /// arguments: If you need an argument to a Java method, set it here.
  /// resultTypeName: If you need to cast a result value passing into callback function, set its name of type here.
  static Future<_CancelListening> _startListening(
      {@required String methodName, @required _ReproCallback callback, List arguments, String resultTypeName}) async {
    int currentListenerId = _nextCallbackId++;

    _callbacksById.add(_CallbackTask(currentListenerId, callback, resultTypeName));

    var args = [];
    args.add(currentListenerId);
    if (arguments != null) {
      args += arguments;
    }
    await _invokeMethod(methodName, args);

    // Returns async function
    // When the callback processing is complete, developers need to execute the function that is returned here.
    return () {
      _invokeMethod('cancelListening', currentListenerId);
      _callbacksById.remove(currentListenerId);
    };
  }

  /// Get the user ID which you set.
  static Future<String> getUserID() async {
    final userID = await _invokeMethod<String>('getUserID');
    return userID;
  }

  /// Set the user ID.
  static Future<void> setUserID(final String userID) async {
    assert(userID != null && userID != null);
    await _invokeMethod('setUserID', userID);
  }

  /// Get the device ID.
  static Future<String> getDeviceID() async {
    final deviceID = await _invokeMethod<String>('getDeviceID');
    return deviceID;
  }

  /// You can change the log level to one of the followings:
  ///  * [LogLevel.debug]
  ///  * [LogLeve.info] (by default)
  ///  * [LogLevel.warn]
  ///  * [LogLevel.error]
  static Future<void> setLogLevel(final LogLevel logLevel) async {
    String logLevelStr;
    switch (logLevel) {
      case LogLevel.debug:
        logLevelStr = 'Debug';
        break;
      case LogLevel.info:
        logLevelStr = 'Info';
        break;
      case LogLevel.warn:
        logLevelStr = 'Warn';
        break;
      case LogLevel.error:
        logLevelStr = 'Error';
        break;
    }
    await _invokeMethod('setLogLevel', logLevelStr);
  }

  /// Set the Cookie for Silver Egg.
  static Future<void> setSilverEggCookie(final String cookie) async {
    await _invokeMethod('setSilverEggCookie', cookie);
  }

  /// Set the ProdKey for Silve Egg.
  static Future<void> setSilverEggProdKey(final String prodKey) async {
    await _invokeMethod('setSilverEggProdKey', prodKey);
  }

  /// Prolong the presentation of in-app messages.
  /// This API must be called before [Repro.setup()].
  static Future<void> disableInAppMessagesOnForegroundTransition() async {
    await _invokeMethod('disableInAppMessagesOnForegroundTransition');
  }

  /// Enable showing the InApp when the app is in foreground.
  /// This API should usually be called when the app's splash screen is complete.
  static Future<void> enableInAppMessagesOnForegroundTransition() async {
    await _invokeMethod('enableInAppMessagesOnForegroundTransition');
  }

  /// Enables or disables the SDK to collect data and display in-app messages.
  static Future<void> optIn(final bool endUserOptedIn) async {
    assert(endUserOptedIn != null && endUserOptedIn != null);
    await _invokeMethod('optIn', endUserOptedIn);
  }

  /// Set a user profile of type String.
  static Future<void> setStringUserProfile(final String key, final String value) async {
    assert(key != null && value != null);
    await _invokeMethod('setStringUserProfile', [key, value]);
  }

  /// Set a user profile of type int.
  static Future<void> setIntUserProfile(final String key, final int value) async {
    assert(key != null && value != null);
    await _invokeMethod('setIntUserProfile', [key, value]);
  }

  /// Set a user profile of type double.
  static Future<void> setDoubleUserProfile(final String key, final double value) async {
    assert(key != null && value != null);
    await _invokeMethod('setDoubleUserProfile', [key, value]);
  }

  /// Set a user profile of type DateTime.
  static Future<void> setDateUserProfile(final String key, final DateTime value) async {
    assert(key != null && value != null);
    await _invokeMethod('setDateUserProfile', [key, value.millisecondsSinceEpoch]);
  }

  /// Set the gender of the user to the user profile.
  static Future<void> setUserGender(final UserGender userGender) async {
    assert(userGender != null);
    String userGenderStr;
    switch (userGender) {
      case UserGender.male:
        userGenderStr = 'Male';
        break;
      case UserGender.female:
        userGenderStr = 'Female';
        break;
      case UserGender.other:
        userGenderStr = 'Other';
        break;
    }
    await _invokeMethod('setUserGender', userGenderStr);
  }

  /// Set the email address of the user to the user profile.
  static Future<void> setUserEmailAddress(final String address) async {
    assert(address != null);
    await _invokeMethod('setUserEmailAddress', address);
  }

  /// Track a custom event.
  static Future<void> track(final String event, [final Map<String, dynamic> properties = const {}]) async {
    assert(event != null);
    await _invokeMethod('track', [event, properties]);
  }

  /// Track the content browsing operation.
  static Future<void> trackViewContent(final String contentId,
      [final Map<String, dynamic> properties = const {}]) async {
    assert(contentId != null);
    await _invokeMethod('trackViewContent', [contentId, properties]);
  }

  /// Track the operation to search.
  static Future<void> trackSearch([final Map<String, dynamic> properties = const {}]) async {
    await _invokeMethod('trackSearch', properties);
  }

  /// Track the operation to add products to the wish list.
  static Future<void> trackAddToWishlist([final Map<String, dynamic> properties = const {}]) async {
    await _invokeMethod('trackAddToWishlist', properties);
  }

  /// Track the operation to add to the cart.
  static Future<void> trackAddToCart(final String contentId, [final Map<String, dynamic> properties = const {}]) async {
    assert(contentId != null);
    await _invokeMethod('trackAddToCart', [contentId, properties]);
  }

  /// Track the start of the purchase process.
  static Future<void> trackInitiateCheckout([final Map<String, dynamic> properties = const {}]) async {
    await _invokeMethod('trackInitiateCheckout', properties);
  }

  /// Track payment information entry operations.
  static Future<void> trackAddPaymentInfo([final Map<String, dynamic> properties = const {}]) async {
    await _invokeMethod('trackAddPaymentInfo', properties);
  }

  /// Track purchase operations.
  static Future<void> trackPurchase(final String contentId, final double value, final String currency,
      [final Map<String, dynamic> properties = const {}]) async {
    assert(contentId != null && value != null && currency != null);
    await _invokeMethod('trackPurchase', [contentId, value, currency, properties]);
  }

  /// Track the operation to share to social media (Facebook, Twitter, etc.).
  static Future<void> trackShare([final Map<String, dynamic> properties = const {}]) async {
    await _invokeMethod('trackShare', properties);
  }

  /// Track the completion of user registration.
  static Future<void> trackCompleteRegistration([final Map<String, dynamic> properties = const {}]) async {
    await _invokeMethod('trackCompleteRegistration', properties);
  }

  /// Starts the trial of the app.
  /// Please use this with apps that do not require user registration at the time of installation.
  static Future<void> trackLead([final Map<String, dynamic> properties = const {}]) async {
    await _invokeMethod('trackLead', properties);
  }

  /// Get a singleton instance of RemoteConfig (UX Optimizer).
  /// When using RemoteConfig's features, call the API via this property.
  static RemoteConfig get remoteConfig {
    return RemoteConfig(_channel);
  }

  /// Get a list of news feeds.
  static Future<List<NewsFeedEntry>> getNewsFeeds({
    @required final int limit,
    final int offsetID,
    @required final NewsFeedCampaignType campaignType}) async
  {
    if (limit == null || limit < 1) {
      throw new ArgumentError("getNewsFeeds: `limit` must be a positive value.");
    }

    if (offsetID != null && offsetID < 1) {
      throw new ArgumentError("getNewsFeeds: `offsetID` must be a positive value or null.");
    }

    if (campaignType == null || campaignType == NewsFeedCampaignType.Unknown) {
      throw new ArgumentError("getNewsFeeds: Invalid value was set for `campaignType`.");
    }

    // int will be converted to Integer or NSNumber.
    var newsFeeds = await _invokeListMethod<Map>('getNewsFeeds', [limit, offsetID, DartNewsFeedUtils.convertCampaignTypeToString(campaignType)]);

    var list = <NewsFeedEntry>[];

    if (newsFeeds != null && newsFeeds.length > 0) {
      for (var newsFeed in newsFeeds) {
        var map = Map<String, String>.from(newsFeed); // Convert Map<dynamic, dynamic> to Map<String, String>
        list.add(NewsFeedEntry(map));
      }
    }
    return list;
  }

  /// Update the read/shown flag of the news feed.
  static Future<void> updateNewsFeeds(List<NewsFeedEntry> newsFeeds) async {
    var jsonList = <String>[];
    var stringMapList = <Map>[];

    for (var entry in newsFeeds) {
      var mapEntry = {
        'newsfeed_id': entry.id,
        'device_id': entry.deviceID,
        'title': entry.title,
        'summary': entry.summary,
        'body': entry.body,
        'link_url': entry.linkUrl,
        'image_url': entry.imageUrl,
        'delivered_at': entry.rawDeliveredAt,
        'shown': entry.shown,
        'read': entry.read
      };

      if (Platform.isAndroid) {
        String json = jsonEncode(mapEntry);
        jsonList.add(json); // Convert JSON string to NewsFeedEntry on the Java side.
      } else if (Platform.isIOS) {
        stringMapList.add(mapEntry);
      }
    }

    if (Platform.isAndroid) {
      await _invokeMethod<String>('updateNewsFeeds', jsonList);
    } else if (Platform.isIOS) {
      await _invokeMethod('updateNewsFeeds', stringMapList);
    }
  }

  static Future<T> _invokeMethod<T>(String method, [dynamic arguments]) async {
    try {
      return await _channel.invokeMethod<T>(method, arguments);
    } catch (e) {
      debugPrint('[Error#ReproFlutterPlugin] $e');
      return null;
    }
  }

  static Future<Map<K, V>> _invokeMapMethod<K, V>(String method, [dynamic arguments]) async {
    try {
      return await _channel.invokeMapMethod<K, V>(method, arguments);
    } catch (e) {
      debugPrint('[Error#ReproFlutterPlugin] $e');
      return null;
    }
  }

  static Future<List<T>> _invokeListMethod<T>(String method, [dynamic arguments]) async {
    try {
      return await _channel.invokeListMethod<T>(method, arguments);
    } catch (e) {
      debugPrint('[Error#ReproFlutterPlugin] $e');
      return null;
    }
  }
}

/// Class for managing Dart's callback function, which is called after calling the Java process.
class _CallbackTask {
  /// ID to identify the callback function.
  int callbackId;

  /// callback function
  _ReproCallback callbackFunction;

  /// The type name of the value to be passed as an argument to the callback function.
  ///
  /// This is used to cast the value of the Java processing result.
  /// If you don't need to do any special casting, you don't need to specify [resultTypeName].
  ///
  /// example: In the following example, the argument of the callback function [result]
  /// is of type [FetchStatus], so the [resultTypeName] should be 'FetchStatus'.
  ///
  /// ```dart
  /// await Repro.remoteConfig.fetch(1000, (result) {
  ///   debugPrint("FetchStatus: ${result.toString()}");
  /// });
  /// ```
  /// In the above example, you need to specify the name of the type to be converted,
  /// in order to convert a value of type int returned by Java to a FetchStatus type.
  String resultTypeName;

  _CallbackTask(this.callbackId, this.callbackFunction, [this.resultTypeName]);
}
